import json
import logging
import sys
import os.path
import uuid
import git
from parsec import ParsecError
from parsec.state import BasicState
from skill_tree.exercises.markdown import parse

from hooks.util import load_json, dump_json, user_name, tree_name, default_notebook

logger = logging.getLogger(__name__)
logger.setLevel(logging.INFO)
handler = logging.StreamHandler(sys.stdout)
formatter = logging.Formatter('%(asctime)s - %(levelname)s - %(message)s')
handler.setFormatter(formatter)
logger.addHandler(handler)


def ensure_node(path, meta):
    if not os.path.exists(path):
        node = {
            "keywords": [],
            "children": [],
            "node_id": f"{tree_name()}-{uuid.uuid4().hex}",
            "export": [meta]}
        dump_json(path, node, exist_ok=True, override=False)
    else:
        node = load_json(path)
        if "node_id" not in node:
            node["node_id"] = f"{tree_name()}-{uuid.uuid4().hex}"
        if meta not in node.get("export", []):
            export = node.get("export", [])
            export.append(meta)
            node["export"] = export
        dump_json(path, node, True, True)


def ensure_doc(path):
    with open(path, encoding="utf-8") as efile:
        try:
            data = efile.read()
        except UnicodeDecodeError:
            logger.error(f"markdown 文件[{path}] 编码错误，请将其以 utf-8 编码保存")
    state = BasicState(data)
    try:
        doc = parse(state)
    except ParsecError as err:
        index = state.index
        context = state.data[index - 15:index + 15]
        logger.error(f"习题 [{path}] 解析失败，在位置 {index} [{context}] 附近有格式: [{err}]")


def ensure_exercise(path):
    if not os.path.exists(path):
        logger.error(f"exercise [{path}] not exists, please check it.")
        sys.exit(1)
    else:
        ensure_doc(path)
        dir, source = os.path.split(path)
        base, ext = os.path.splitext(path)
        meta_path = base + ".json"
        _, mfile = os.path.split(meta_path)
        if os.path.exists(meta_path):
            meta = load_json(meta_path)
            if "exercise_id" not in meta:
                meta["exercise_id"] = uuid.uuid4().hex
            if "notebook_enable" not in meta:
                meta["notebook_enable"] = default_notebook()
            if "source" not in meta:
                meta["source"] = source
            if "author" not in meta:
                meta["author"] = user_name()
            if "type" not in meta:
                meta["type"] = "code_options"
        else:
            meta = {
                "type": "code_options",
                "author": user_name(),
                "source": source,
                "notebook_enable": default_notebook(),
                "exercise_id": uuid.uuid4().hex
            }

        dump_json(meta_path, meta, True, True)
        node_path = os.path.join(dir, "config.json")
        ensure_node(node_path, mfile)


def check_section_node(path):
    dir, _ = os.path.split(path)
    data = load_json(path)
    if not data.get("keywords", []):
        logger.error(f"[{path}] 的 keywords 字段为空，请填入适当的关键字")
        sys.exit(1)


def pre_commit():
    path = sys.argv[1]
    _, ext = os.path.splitext(path)
    _, filename = os.path.split(path)
    if ext == ".md" and path.startswith("data/"):
        ensure_exercise(path)
    elif filename == "config.json" and path.startswith("data/") and path.count("/") == 4:
        check_section_node(path)
